﻿using System;
namespace Persistence {

  #region Serialization Tokens

  public interface ISerializationToken {
  }
  public class BeginToken : ISerializationToken {
    public readonly DataMemberAccessor Accessor;
    public readonly int Id;
    public BeginToken(DataMemberAccessor accessor, int id) {
      Accessor = accessor;
      Id = id;
    }
  }
  public class EndToken : ISerializationToken {
    public readonly DataMemberAccessor Accessor;
    public readonly int Id;
    public EndToken(DataMemberAccessor accessor, int id) {
      Accessor = accessor;
      Id = id;
    }
  }
  public class PrimitiveToken : ISerializationToken {
    public readonly DataMemberAccessor Accessor;
    public readonly object Data;
    public readonly int? Id;
    public PrimitiveToken(DataMemberAccessor accessor, object data, int? id) {
      Accessor = accessor;
      Data = data;
      Id = id;
    }
  }
  public class ReferenceToken : ISerializationToken {
    public readonly string Name;
    public readonly int Id;
    public ReferenceToken(string name, int id) {
      Name = name;
      Id = id;
    }
  }
  public class NullReferenceToken : ISerializationToken {
    public readonly string Name;
    public NullReferenceToken(string name) {
      Name = name;
    }
  }
  #endregion

  #region DeSerialization Tokens

  public interface IParseToken { }

  public class CompositeStart : IParseToken {
    public readonly string Name;
    public readonly Type Type;
    public readonly int Id;
    public CompositeStart(string name, Type type, int id) {
      Name = name;
      Type = type;
      Id = id;
    }
  }
  public class CompositeEnd : IParseToken {
    public readonly string Name;
    public readonly Type Type;
    public readonly int Id;
    public CompositeEnd(string name, Type type, int id) {
      Name = name;
      Type = type;
      Id = id;
    }
  }
  public class Primitive : IParseToken {
    public readonly string Name;
    public readonly Type Type;
    public readonly string SerializedValue;
    public readonly int? Id;
    public Primitive(string name, Type type, string serilaizedValue, int? id) {
      Name = name;
      Type = type;
      SerializedValue = serilaizedValue;
      Id = id;
    }
  }
  public class Reference : IParseToken {
    public readonly string Name;
    public readonly int Id;
    public Reference(string name, int id) {
      Name = name;
      Id = id;
    }
  }
  public class Null : IParseToken {
    public readonly string Name;
    public Null(string name) {
      Name = name;
    }
  }
  #endregion

}